@@ -73,6 +73,8 @@ def object_from_json(json):
         return OvsPatchPort.from_json(json)
     elif obj_type == "ib_interface":
         return IbInterface.from_json(json)
+    elif obj_type == "ib_child_interface":
+        return IbChildInterface.from_json(json)
     elif obj_type == "ovs_dpdk_port":
         return OvsDpdkPort.from_json(json)
     elif obj_type == "ovs_dpdk_bond":
@@ -463,7 +465,8 @@ class Interface(_BaseOpts):
                  routes=None, rules=None, mtu=None, primary=False,
                  nic_mapping=None, persist_mapping=False, defroute=True,
                  dhclient_args=None, dns_servers=None, nm_controlled=False,
-                 onboot=True, domain=None, ethtool_opts=None, hotplug=False):
+                 onboot=True, domain=None, ethtool_opts=None, hotplug=False,
+                 linkdelay=None):
         addresses = addresses or []
         routes = routes or []
         rules = rules or []
@@ -475,6 +478,7 @@ class Interface(_BaseOpts):
                                         nm_controlled, onboot, domain)
         self.ethtool_opts = ethtool_opts
         self.hotplug = hotplug
+        self.linkdelay = linkdelay
 
     @staticmethod
     def from_json(json):
@@ -482,8 +486,9 @@ class Interface(_BaseOpts):
         hotplug = strutils.bool_from_string(str(json.get('hotplug', False)))
         opts = _BaseOpts.base_opts_from_json(json)
         ethtool_opts = json.get('ethtool_opts', None)
+        linkdelay = json.get('linkdelay', None)
         return Interface(name, *opts, ethtool_opts=ethtool_opts,
-                         hotplug=hotplug)
+                         hotplug=hotplug, linkdelay=linkdelay)
 
 
 class Vlan(_BaseOpts):
@@ -653,7 +658,10 @@ class OvsBridge(_BaseOpts):
                                   vlan_id=iface.vlan_id, qos=iface.qos,
                                   spoofcheck=iface.spoofcheck,
                                   trust=iface.trust, state=iface.state,
-                                  macaddr=iface.macaddr, promisc=iface.promisc)
+                                  macaddr=iface.macaddr, promisc=iface.promisc,
+                                  pci_address=iface.pci_address,
+                                  min_tx_rate=iface.min_tx_rate,
+                                  max_tx_rate=iface.max_tx_rate)
 
     @staticmethod
     def from_json(json):
@@ -975,7 +983,7 @@ class LinuxBond(_BaseOpts):
                  members=None, bonding_options=None, nic_mapping=None,
                  persist_mapping=False, defroute=True, dhclient_args=None,
                  dns_servers=None, nm_controlled=False, onboot=True,
-                 domain=None):
+                 domain=None, ethtool_opts=None):
         addresses = addresses or []
         routes = routes or []
         rules = rules or []
@@ -988,6 +996,7 @@ class LinuxBond(_BaseOpts):
                                         nm_controlled, onboot, domain)
         self.members = members
         self.bonding_options = bonding_options
+        self.ethtool_opts = ethtool_opts
         for member in self.members:
             if isinstance(member, SriovVF):
                 LinuxBond.update_vf_config(member)
@@ -1019,7 +1028,10 @@ class LinuxBond(_BaseOpts):
                                   vlan_id=iface.vlan_id, qos=iface.qos,
                                   spoofcheck=iface.spoofcheck,
                                   trust=iface.trust, state=iface.state,
-                                  macaddr=iface.macaddr, promisc=iface.promisc)
+                                  macaddr=iface.macaddr, promisc=iface.promisc,
+                                  pci_address=iface.pci_address,
+                                  min_tx_rate=iface.min_tx_rate,
+                                  max_tx_rate=iface.max_tx_rate)
 
     @staticmethod
     def from_json(json):
@@ -1029,6 +1041,7 @@ class LinuxBond(_BaseOpts):
          onboot, domain) = _BaseOpts.base_opts_from_json(
              json, include_primary=False)
         bonding_options = json.get('bonding_options')
+        ethtool_opts = json.get('ethtool_opts', None)
 
         members = _update_members(json, nic_mapping, persist_mapping)
 
@@ -1040,7 +1053,7 @@ class LinuxBond(_BaseOpts):
                          persist_mapping=persist_mapping, defroute=defroute,
                          dhclient_args=dhclient_args, dns_servers=dns_servers,
                          nm_controlled=nm_controlled, onboot=onboot,
-                         domain=domain)
+                         domain=domain, ethtool_opts=ethtool_opts)
 
 
 class OvsBond(_BaseOpts):
@@ -1102,7 +1115,9 @@ class OvsBond(_BaseOpts):
                                   vlan_id=iface.vlan_id, qos=iface.qos,
                                   spoofcheck=iface.spoofcheck,
                                   trust=iface.trust, state=iface.state,
-                                  macaddr=iface.macaddr, promisc=iface.promisc)
+                                  macaddr=iface.macaddr, promisc=iface.promisc,
+                                  min_tx_rate=iface.min_tx_rate,
+                                  max_tx_rate=iface.max_tx_rate)
 
     @staticmethod
     def from_json(json):
@@ -1237,6 +1252,55 @@ class IbInterface(_BaseOpts):
         return IbInterface(name, *opts, ethtool_opts=ethtool_opts)
 
 
+class IbChildInterface(_BaseOpts):
+    """Base class for InfiniBand child network interfaces."""
+
+    def __init__(self, parent, pkey_id, use_dhcp=False, use_dhcpv6=False,
+                 addresses=None, routes=None, rules=None, mtu=None,
+                 primary=False, nic_mapping=None, persist_mapping=False,
+                 defroute=True, dhclient_args=None, dns_servers=None,
+                 nm_controlled=True, onboot=True, domain=None):
+        addresses = addresses or []
+        routes = routes or []
+        rules = rules or []
+        dns_servers = dns_servers or []
+        self.pkey_id = pkey_id
+        self.parent = parent
+        full_pkey_id = 0x8000 | pkey_id
+        name = "%s.%04x" % (parent, full_pkey_id)
+        nm_controlled = True
+        super(IbChildInterface, self).__init__(name, use_dhcp, use_dhcpv6,
+                                               addresses, routes, rules, mtu,
+                                               primary, nic_mapping,
+                                               persist_mapping, defroute,
+                                               dhclient_args, dns_servers,
+                                               nm_controlled, onboot, domain)
+
+    @staticmethod
+    def from_json(json):
+        parent = _get_required_field(json, 'parent', 'IbChildInterface')
+        pkey_id = _get_required_field(json, 'pkey_id', 'IbChildInterface')
+        if type(pkey_id) == str:
+            try:
+                pkey_id = int(pkey_id)
+            except ValueError:
+                try:
+                    pkey_id = int(pkey_id, base=16)
+                except Exception:
+                    # Note (Abdallahyas): We do not care for other
+                    # bases other than decimal and hexa
+                    msg = "pkey only support decimal and hex bases, not int"
+                    raise InvalidConfigException(msg)
+        else:
+            msg = "Invalid pkey type: got %s" % type(pkey_id)
+            raise InvalidConfigException(msg)
+        if (pkey_id < 0x0001 or pkey_id >= 0x7fff):
+            msg = "Invalid pkey value 0x%X" % pkey_id
+            raise InvalidConfigException(msg)
+        opts = _BaseOpts.base_opts_from_json(json)
+        return IbChildInterface(parent, pkey_id, *opts)
+
+
 class OvsDpdkPort(_BaseOpts):
     """Base class for OVS Dpdk Ports."""
 
@@ -1278,7 +1342,10 @@ class OvsDpdkPort(_BaseOpts):
                                   vlan_id=iface.vlan_id, qos=iface.qos,
                                   spoofcheck=iface.spoofcheck,
                                   trust=iface.trust, state=iface.state,
-                                  macaddr=iface.macaddr, promisc=iface.promisc)
+                                  macaddr=iface.macaddr, promisc=iface.promisc,
+                                  pci_address=iface.pci_address,
+                                  min_tx_rate=iface.min_tx_rate,
+                                  max_tx_rate=iface.max_tx_rate)
 
     @staticmethod
     def from_json(json):
@@ -1352,7 +1419,7 @@ class SriovVF(_BaseOpts):
                  defroute=True, dhclient_args=None, dns_servers=None,
                  nm_controlled=False, onboot=True, domain=None, vlan_id=0,
                  qos=0, spoofcheck=None, trust=None, state=None, macaddr=None,
-                 promisc=None):
+                 promisc=None, min_tx_rate=0, max_tx_rate=0):
         addresses = addresses or []
         routes = routes or []
         rules = rules or []
@@ -1373,11 +1440,17 @@ class SriovVF(_BaseOpts):
         self.device = device
         self.vlan_id = int(vlan_id)
         self.qos = int(qos)
+        self.min_tx_rate = int(min_tx_rate)
+        self.max_tx_rate = int(max_tx_rate)
         self.spoofcheck = spoofcheck
         self.trust = trust
         self.state = state
+        pci_address = utils.get_pci_address(name, False)
+        if pci_address is None:
+            pci_address = utils.get_stored_pci_address(name, False)
         self.macaddr = macaddr
         self.promisc = promisc
+        self.pci_address = pci_address
         utils.update_sriov_vf_map(device, self.vfid, name,
                                   vlan_id=self.vlan_id,
                                   qos=self.qos,
@@ -1385,7 +1458,10 @@ class SriovVF(_BaseOpts):
                                   trust=trust,
                                   state=state,
                                   macaddr=macaddr,
-                                  promisc=promisc)
+                                  promisc=promisc,
+                                  pci_address=pci_address,
+                                  min_tx_rate=min_tx_rate,
+                                  max_tx_rate=max_tx_rate)
 
     @staticmethod
     def get_on_off(config):
@@ -1408,6 +1484,13 @@ class SriovVF(_BaseOpts):
         if qos != 0 and vlan_id == 0:
             msg = "Vlan tag not set for QOS - VF: %s:%d" % (device, vfid)
             raise InvalidConfigException(msg)
+        min_tx_rate = json.get('min_tx_rate', 0)
+        max_tx_rate = json.get('max_tx_rate', 0)
+        if max_tx_rate > 0 and min_tx_rate > max_tx_rate:
+            msg = ("vf %s:%d: min_tx_rate(%d) > max_tx_rate(%d)" %
+                   (device, vfid, min_tx_rate, max_tx_rate))
+            raise InvalidConfigException(msg)
+
         spoofcheck = SriovVF.get_on_off(json.get('spoofcheck'))
         trust = SriovVF.get_on_off(json.get('trust'))
         promisc = SriovVF.get_on_off(json.get('promisc'))
@@ -1418,7 +1501,8 @@ class SriovVF(_BaseOpts):
         macaddr = json.get('macaddr')
         return SriovVF(device, vfid, *opts, vlan_id=vlan_id, qos=qos,
                        spoofcheck=spoofcheck, trust=trust, state=state,
-                       macaddr=macaddr, promisc=promisc)
+                       macaddr=macaddr, promisc=promisc,
+                       min_tx_rate=min_tx_rate, max_tx_rate=max_tx_rate)
 
 
 class SriovPF(_BaseOpts):
