@@ -160,6 +160,12 @@ class TestInterface(base.TestCase):
         self.assertEqual("em1", interface.name)
         self.assertTrue(interface.use_dhcp)
 
+    def test_from_json_dhcpv6(self):
+        data = '{"type": "interface", "name": "em1", "use_dhcpv6": true}'
+        interface = objects.object_from_json(json.loads(data))
+        self.assertEqual("em1", interface.name)
+        self.assertTrue(interface.use_dhcpv6)
+
     def test_from_json_dotted_vlan(self):
         def dummy_mapped_nics(nic_mapping=None):
             return {"nic1": "em3"}
@@ -456,13 +462,20 @@ class TestBridge(base.TestCase):
         vf_final = [{'device_type': 'vf', 'name': 'em1_1',
                      'device': {'name': 'em1', 'vfid': 1},
                      'vlan_id': 111, 'qos': 1,
+                     'min_tx_rate': 0, 'max_tx_rate': 0,
                      'spoofcheck': 'off', 'trust': 'on',
-                     'promisc': 'on'}]
+                     'promisc': 'on', 'pci_address': '0000:79:10.2'}]
 
         def test_get_vf_devname(device, vfid):
             return device + '_' + str(vfid)
+
+        def test_get_pci_address(ifname, noop):
+            return '0000:79:10.2'
+
         self.stub_out('os_net_config.utils.get_vf_devname',
                       test_get_vf_devname)
+        self.stub_out('os_net_config.utils.get_pci_address',
+                      test_get_pci_address)
 
         bridge = objects.object_from_json(json.loads(data))
         self.assertEqual("br-foo", bridge.name)
@@ -506,18 +519,26 @@ class TestBridge(base.TestCase):
         vf_final = [{'device_type': 'vf', 'name': 'em1_1',
                      'device': {'name': 'em1', 'vfid': 1},
                      'vlan_id': 111, 'qos': 1,
+                     'min_tx_rate': 0, 'max_tx_rate': 0,
                      'spoofcheck': 'off', 'trust': 'on',
                      'promisc': 'on'},
                     {'device_type': 'vf', 'name': 'em2_1',
                      'device': {'name': 'em2', 'vfid': 1},
                      'vlan_id': 111, 'qos': 1,
+                     'min_tx_rate': 0, 'max_tx_rate': 0,
                      'spoofcheck': 'off', 'trust': 'on',
                      'promisc': 'on'}]
 
         def test_get_vf_devname(device, vfid):
             return device + '_' + str(vfid)
+
+        def test_get_pci_address(ifname, noop):
+            return '0000:79:10.2'
+
         self.stub_out('os_net_config.utils.get_vf_devname',
                       test_get_vf_devname)
+        self.stub_out('os_net_config.utils.get_pci_address',
+                      test_get_pci_address)
 
         bridge = objects.object_from_json(json.loads(data))
         self.assertEqual("br-foo", bridge.name)
@@ -569,18 +590,26 @@ class TestBridge(base.TestCase):
         vf_final = [{'device_type': 'vf', 'name': 'em1_1',
                      'device': {'name': 'em1', 'vfid': 1},
                      'vlan_id': 111, 'qos': 1,
+                     'min_tx_rate': 0, 'max_tx_rate': 0,
                      'spoofcheck': 'on', 'trust': 'off',
                      'promisc': 'off'},
                     {'device_type': 'vf', 'name': 'em2_1',
                      'device': {'name': 'em2', 'vfid': 1},
                      'vlan_id': 111, 'qos': 1,
+                     'min_tx_rate': 0, 'max_tx_rate': 0,
                      'spoofcheck': 'on', 'trust': 'off',
                      'promisc': 'off'}]
 
         def test_get_vf_devname(device, vfid):
             return device + '_' + str(vfid)
+
+        def test_get_pci_address(ifname, noop):
+            return '0000:79:10.2'
+
         self.stub_out('os_net_config.utils.get_vf_devname',
                       test_get_vf_devname)
+        self.stub_out('os_net_config.utils.get_pci_address',
+                      test_get_pci_address)
 
         objects.object_from_json(json.loads(data))
 
@@ -608,14 +637,21 @@ class TestBridge(base.TestCase):
         vf_final = [{'device_type': 'vf', 'name': 'em1_1',
                      'device': {'name': 'em1', 'vfid': 1},
                      'vlan_id': 111, 'qos': 1,
+                     'min_tx_rate': 0, 'max_tx_rate': 0,
+                     'pci_address': '0000:79:10.2',
                      'spoofcheck': 'off', 'trust': 'off',
                      'promisc': 'off'}]
 
         def test_get_vf_devname(device, vfid):
             return device + '_' + str(vfid)
+
+        def test_get_pci_address(ifname, noop):
+            return '0000:79:10.2'
+
         self.stub_out('os_net_config.utils.get_vf_devname',
                       test_get_vf_devname)
-
+        self.stub_out('os_net_config.utils.get_pci_address',
+                      test_get_pci_address)
         bridge = objects.object_from_json(json.loads(data))
         self.assertEqual("br-foo", bridge.name)
         self.assertTrue(bridge.use_dhcp)
@@ -651,13 +687,21 @@ class TestBridge(base.TestCase):
         vf_final = [{'device_type': 'vf', 'name': 'em1_1',
                      'device': {'name': 'em1', 'vfid': 1},
                      'vlan_id': 111, 'qos': 1,
-                     'spoofcheck': 'off', 'trust': 'on'
+                     'min_tx_rate': 0, 'max_tx_rate': 0,
+                     'spoofcheck': 'off', 'trust': 'on',
+                     'pci_address': '0000:79:10.2'
                      }]
 
         def test_get_vf_devname(device, vfid):
             return device + '_' + str(vfid)
+
+        def test_get_pci_address(ifname, noop):
+            return '0000:79:10.2'
+
         self.stub_out('os_net_config.utils.get_vf_devname',
                       test_get_vf_devname)
+        self.stub_out('os_net_config.utils.get_pci_address',
+                      test_get_pci_address)
 
         bridge = objects.object_from_json(json.loads(data))
         self.assertEqual("br-foo", bridge.name)
@@ -686,6 +730,8 @@ class TestBridge(base.TestCase):
                 "vfid": 1,
                 "vlan_id": 111,
                 "qos": 1,
+                "min_tx_rate": 100,
+                "max_tx_rate": 500,
                 "spoofcheck": false,
                 "trust": false,
                 "promisc": false
@@ -697,13 +743,21 @@ class TestBridge(base.TestCase):
         vf_final = [{'device_type': 'vf', 'name': 'em1_1',
                      'device': {'name': 'em1', 'vfid': 1},
                      'vlan_id': 111, 'qos': 1,
-                     'spoofcheck': 'off', 'trust': 'off'
+                     'min_tx_rate': 100, 'max_tx_rate': 500,
+                     'spoofcheck': 'off', 'trust': 'off',
+                     'pci_address': '0000:79:10.2'
                      }]
 
         def test_get_vf_devname(device, vfid):
             return device + '_' + str(vfid)
+
+        def test_get_pci_address(ifname, noop):
+            return '0000:79:10.2'
+
         self.stub_out('os_net_config.utils.get_vf_devname',
                       test_get_vf_devname)
+        self.stub_out('os_net_config.utils.get_pci_address',
+                      test_get_pci_address)
 
         bridge = objects.object_from_json(json.loads(data))
         self.assertEqual("br-foo", bridge.name)
@@ -1135,12 +1189,13 @@ class TestLinuxBond(base.TestCase):
 ]
 }
 """
-        bridge = objects.object_from_json(json.loads(data))
-        self.assertEqual("bond1", bridge.name)
-        self.assertTrue(bridge.use_dhcp)
-        interface1 = bridge.members[0]
+        bond = objects.object_from_json(json.loads(data))
+        self.assertEqual("bond1", bond.name)
+        self.assertTrue(bond.use_dhcp)
+        self.assertIsNone(bond.ethtool_opts)
+        interface1 = bond.members[0]
         self.assertEqual("em1", interface1.name)
-        interface2 = bridge.members[1]
+        interface2 = bond.members[1]
         self.assertEqual("em2", interface2.name)
 
     def test_from_json_dhcp_with_nic1_nic2(self):
@@ -1165,12 +1220,12 @@ class TestLinuxBond(base.TestCase):
 ]
 }
 """
-        bridge = objects.object_from_json(json.loads(data))
-        self.assertEqual("bond1", bridge.name)
-        self.assertTrue(bridge.use_dhcp)
-        interface1 = bridge.members[0]
+        bond = objects.object_from_json(json.loads(data))
+        self.assertEqual("bond1", bond.name)
+        self.assertTrue(bond.use_dhcp)
+        interface1 = bond.members[0]
         self.assertEqual("em1", interface1.name)
-        interface2 = bridge.members[1]
+        interface2 = bond.members[1]
         self.assertEqual("em2", interface2.name)
 
     def test_linux_bond_with_vf_default(self):
@@ -1184,6 +1239,7 @@ class TestLinuxBond(base.TestCase):
     "vfid": 1,
     "vlan_id": 111,
     "qos": 1,
+    "max_tx_rate": 10,
     "primary": true
     },
     {
@@ -1191,25 +1247,39 @@ class TestLinuxBond(base.TestCase):
     "device": "em2",
     "vfid": 1,
     "vlan_id": 111,
-    "qos": 1
+    "qos": 1,
+    "max_tx_rate": 10
 }]
 }
 """
         vf_final = [{'device_type': 'vf', 'name': 'em1_1',
                      'device': {'name': 'em1', 'vfid': 1},
                      'vlan_id': 111, 'qos': 1,
+                     'min_tx_rate': 0, 'max_tx_rate': 10,
+                     'pci_address': '0000:79:10.1',
                      'spoofcheck': 'on', 'trust': 'on',
                      'promisc': 'off'},
                     {'device_type': 'vf', 'name': 'em2_1',
                      'device': {'name': 'em2', 'vfid': 1},
                      'vlan_id': 111, 'qos': 1,
+                     'min_tx_rate': 0, 'max_tx_rate': 10,
+                     'pci_address': '0000:79:10.2',
                      'spoofcheck': 'on', 'trust': 'on',
                      'promisc': 'off'}]
 
         def test_get_vf_devname(device, vfid):
             return device + '_' + str(vfid)
+
+        def test_get_pci_address(ifname, noop):
+            if ifname == 'em1_1':
+                return '0000:79:10.1'
+            elif ifname == 'em2_1':
+                return '0000:79:10.2'
+
         self.stub_out('os_net_config.utils.get_vf_devname',
                       test_get_vf_devname)
+        self.stub_out('os_net_config.utils.get_pci_address',
+                      test_get_pci_address)
 
         bond = objects.object_from_json(json.loads(data))
         self.assertEqual("bond1", bond.name)
@@ -1254,24 +1324,70 @@ class TestLinuxBond(base.TestCase):
         vf_final = [{'device_type': 'vf', 'name': 'em1_1',
                      'device': {'name': 'em1', 'vfid': 1},
                      'vlan_id': 111, 'qos': 1,
+                     'min_tx_rate': 0, 'max_tx_rate': 0,
+                     'pci_address': '0000:79:10.1',
                      'spoofcheck': 'off', 'trust': 'off',
                      'promisc': 'off'},
                     {'device_type': 'vf', 'name': 'em2_1',
                      'device': {'name': 'em2', 'vfid': 1},
                      'vlan_id': 111, 'qos': 1,
+                     'min_tx_rate': 0, 'max_tx_rate': 0,
+                     'pci_address': '0000:79:10.2',
                      'spoofcheck': 'off', 'trust': 'off',
                      'promisc': 'off'}]
 
         def test_get_vf_devname(device, vfid):
             return device + '_' + str(vfid)
+
+        def test_get_pci_address(ifname, noop):
+            if ifname == 'em1_1':
+                return '0000:79:10.1'
+            elif ifname == 'em2_1':
+                return '0000:79:10.2'
+
         self.stub_out('os_net_config.utils.get_vf_devname',
                       test_get_vf_devname)
+        self.stub_out('os_net_config.utils.get_pci_address',
+                      test_get_pci_address)
 
         objects.object_from_json(json.loads(data))
         contents = utils.get_file_data(sriov_config._SRIOV_CONFIG_FILE)
         vf_map = yaml.safe_load(contents) if contents else []
         self.assertListEqual(vf_final, vf_map)
 
+    def test_linux_bond_with_ethtool_opts(self):
+        data = """{
+"type": "linux_bond",
+"name": "bond1",
+"use_dhcp": true,
+"ethtool_opts": "-K ${DEVICE} tx-gre-csum-segmentation off",
+"members": [
+    {
+    "type": "interface",
+    "name": "em1",
+    "ethtool_opts": "-K ${DEVICE} tx-gre-csum-segmentation off"
+    },
+    {
+    "type": "interface",
+    "name": "em2",
+    "ethtool_opts": "-K ${DEVICE} tx-gre-csum-segmentation off"
+    }
+]
+}
+"""
+        bond = objects.object_from_json(json.loads(data))
+        self.assertEqual("bond1", bond.name)
+        self.assertEqual("-K ${DEVICE} tx-gre-csum-segmentation off",
+                         bond.ethtool_opts)
+        interface1 = bond.members[0]
+        interface2 = bond.members[1]
+        self.assertEqual("em1", interface1.name)
+        self.assertEqual("em2", interface2.name)
+        self.assertEqual("-K ${DEVICE} tx-gre-csum-segmentation off",
+                         interface1.ethtool_opts)
+        self.assertEqual("-K ${DEVICE} tx-gre-csum-segmentation off",
+                         interface2.ethtool_opts)
+
 
 class TestOvsTunnel(base.TestCase):
 
@@ -1497,6 +1613,67 @@ class TestIbInterface(base.TestCase):
         self.assertEqual("metric 10", route1.route_options)
 
 
+class TestIbChildInterface(base.TestCase):
+
+    def test_ib_child_interface_name(self):
+        ib_child_interface = objects.IbChildInterface('foo', pkey_id=100)
+        self.assertEqual("foo.8064", ib_child_interface.name)
+        self.assertEqual("foo", ib_child_interface.parent)
+
+    def test_ib_child_interface_pkey_id_valid(self):
+        ib_child_interface = objects.IbChildInterface('foo', pkey_id=100)
+        self.assertEqual(100, ib_child_interface.pkey_id)
+
+    def test_ib_child_interface_pkek_id_str(self):
+        data = '{"type": "ib_child_interface", ' \
+               '"parent": "foo", "pkey_id": "100"}'
+        ib_child_interface = \
+            objects.IbChildInterface.from_json(json.loads(data))
+        self.assertEqual(100, ib_child_interface.pkey_id)
+
+    def test_ib_child_interface_pkey_id_str_hexa(self):
+        data = '{"type": "ib_child_interface", ' \
+               '"parent": "foo", "pkey_id": "0x64"}'
+        ib_child_interface = \
+            objects.IbChildInterface.from_json(json.loads(data))
+        self.assertEqual(100, ib_child_interface.pkey_id)
+
+    def test_ib_child_interface_pkey_id_str_invalid_base(self):
+        data = '{"type": "ib_child_interface", ' \
+               '"parent": "foo", "pkey_id": "0b01100100"}'
+        self.assertRaises(objects.InvalidConfigException,
+                          objects.object_from_json,
+                          json.loads(data))
+
+    def test_ib_child_interface_nm_controlled_true(self):
+        data = '{"type": "ib_child_interface", ' \
+               '"parent": "foo", "pkey_id": "100"}'
+        ib_child_interface = \
+            objects.IbChildInterface.from_json(json.loads(data))
+        self.assertEqual(True, ib_child_interface.nm_controlled)
+
+    def test_ib_child_interface_pkey_id_zero(self):
+        data = '{"type": "ib_child_interface", ' \
+               '"parent": "foo", "pkey_id": "0"}'
+        self.assertRaises(objects.InvalidConfigException,
+                          objects.object_from_json,
+                          json.loads(data))
+
+    def test_ib_child_interface_pkey_id_managment(self):
+        data = '{"type": "ib_child_interface", ' \
+               '"parent": "foo", "pkey_id": "0x7fff"}'
+        self.assertRaises(objects.InvalidConfigException,
+                          objects.object_from_json,
+                          json.loads(data))
+
+    def test_ib_child_interface_pkey_id_max(self):
+        data = '{"type": "ib_child_interface", ' \
+               '"parent": "foo", "pkey_id": "0x8001"}'
+        self.assertRaises(objects.InvalidConfigException,
+                          objects.object_from_json,
+                          json.loads(data))
+
+
 class TestNicMapping(base.TestCase):
 
     # We want to test the function, not the dummy..
@@ -1881,8 +2058,14 @@ class TestSriovVF(base.TestCase):
     def test_from_json_vfid(self):
         def test_get_vf_devname(device, vfid):
             return device + '_' + str(vfid)
+
+        def test_get_pci_address(ifname, noop):
+            return '0000:79:10.2'
+
         self.stub_out('os_net_config.utils.get_vf_devname',
                       test_get_vf_devname)
+        self.stub_out('os_net_config.utils.get_pci_address',
+                      test_get_pci_address)
         data = '{"type": "sriov_vf", "device": "em1", "vfid": 16,' \
                '"use_dhcp": false}'
         vf = objects.object_from_json(json.loads(data))
@@ -1894,8 +2077,14 @@ class TestSriovVF(base.TestCase):
     def test_from_json_name_ignored(self):
         def test_get_vf_devname(device, vfid):
             return device + '_' + str(vfid)
+
+        def test_get_pci_address(ifname, noop):
+            return '0000:79:10.2'
+
         self.stub_out('os_net_config.utils.get_vf_devname',
                       test_get_vf_devname)
+        self.stub_out('os_net_config.utils.get_pci_address',
+                      test_get_pci_address)
         data = '{"type": "sriov_vf", "device": "em1", "vfid": 16,' \
                '"use_dhcp": false, "name": "em1_7"}'
         vf = objects.object_from_json(json.loads(data))
@@ -1907,8 +2096,14 @@ class TestSriovVF(base.TestCase):
     def test_from_json_vfid_configs_enabled(self):
         def test_get_vf_devname(device, vfid):
             return device + '_' + str(vfid)
+
+        def test_get_pci_address(ifname, noop):
+            return '0000:79:10.2'
+
         self.stub_out('os_net_config.utils.get_vf_devname',
                       test_get_vf_devname)
+        self.stub_out('os_net_config.utils.get_pci_address',
+                      test_get_pci_address)
 
         data = '{"type": "sriov_vf", "device": "em4", "vfid": 16,' \
                '"use_dhcp": false, "vlan_id": 100, "qos": 2, "trust": true,' \
@@ -1930,8 +2125,14 @@ class TestSriovVF(base.TestCase):
     def test_from_json_vfid_configs_disabled(self):
         def test_get_vf_devname(device, vfid):
             return device + '_' + str(vfid)
+
+        def test_get_pci_address(ifname, noop):
+            return '0000:79:10.2'
+
         self.stub_out('os_net_config.utils.get_vf_devname',
                       test_get_vf_devname)
+        self.stub_out('os_net_config.utils.get_pci_address',
+                      test_get_pci_address)
 
         data = '{"type": "sriov_vf", "device": "em4", "vfid": 16,' \
                '"use_dhcp": false, "vlan_id": 0, "qos": 0, "trust": false,' \
@@ -1953,8 +2154,14 @@ class TestSriovVF(base.TestCase):
     def test_from_json_vfid_invalid_state(self):
         def test_get_vf_devname(device, vfid):
             return device + '_' + str(vfid)
+
+        def test_get_pci_address(ifname, noop):
+            return '0000:79:10.2'
+
         self.stub_out('os_net_config.utils.get_vf_devname',
                       test_get_vf_devname)
+        self.stub_out('os_net_config.utils.get_pci_address',
+                      test_get_pci_address)
 
         data = '{"type": "sriov_vf", "device": "em4", "vfid": 16,' \
                '"use_dhcp": false, "vlan_id": 0, "qos": 0, "trust": false,' \
@@ -1969,8 +2176,14 @@ class TestSriovVF(base.TestCase):
     def test_from_json_vfid_invalid_qos(self):
         def test_get_vf_devname(device, vfid):
             return device + '_' + str(vfid)
+
+        def test_get_pci_address(ifname, noop):
+            return '0000:79:10.2'
+
         self.stub_out('os_net_config.utils.get_vf_devname',
                       test_get_vf_devname)
+        self.stub_out('os_net_config.utils.get_pci_address',
+                      test_get_pci_address)
 
         data = '{"type": "sriov_vf", "device": "em4", "vfid": 16,' \
                '"use_dhcp": false, "vlan_id": 0, "qos": 10, "trust": false,' \
@@ -1984,8 +2197,14 @@ class TestSriovVF(base.TestCase):
     def test_from_json_vfid_nic1(self):
         def test_get_vf_devname(device, vfid):
             return device + '_' + str(vfid)
+
+        def test_get_pci_address(ifname, noop):
+            return '0000:79:10.2'
+
         self.stub_out('os_net_config.utils.get_vf_devname',
                       test_get_vf_devname)
+        self.stub_out('os_net_config.utils.get_pci_address',
+                      test_get_pci_address)
 
         def dummy_mapped_nics(nic_mapping=None):
             return {"nic1": "em4"}
@@ -2000,6 +2219,30 @@ class TestSriovVF(base.TestCase):
         self.assertFalse(vf.use_dhcp)
         self.assertEqual("em4_16", vf.name)
 
+    def test_from_json_pci_address_none(self):
+        def test_get_vf_devname(device, vfid):
+            return device + '_' + str(vfid)
+
+        def test_get_pci_address(ifname, noop):
+            return None
+
+        def test_get_stored_pci_address(ifname, noop):
+            return '0000:79:10.2'
+
+        self.stub_out('os_net_config.utils.get_vf_devname',
+                      test_get_vf_devname)
+        self.stub_out('os_net_config.utils.get_pci_address',
+                      test_get_pci_address)
+        self.stub_out('os_net_config.utils.get_stored_pci_address',
+                      test_get_stored_pci_address)
+        data = '{"type": "sriov_vf", "device": "em1", "vfid": 16,' \
+               '"use_dhcp": false}'
+        vf = objects.object_from_json(json.loads(data))
+        self.assertEqual("em1", vf.device)
+        self.assertEqual(16, vf.vfid)
+        self.assertFalse(vf.use_dhcp)
+        self.assertEqual("em1_16", vf.name)
+
 
 class TestOvsDpdkBond(base.TestCase):
 
